<?php

namespace Drupal\datetime_flatpickr_bef\Plugin\better_exposed_filters\filter;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\datetime_flatpickr\Plugin\Field\FieldWidget\DateTimeFlatPickrWidgetTrait;
use Drupal\better_exposed_filters\Plugin\better_exposed_filters\filter\FilterWidgetBase;

/**
 * Date picker widget implementation.
 *
 * @BetterExposedFiltersFilterWidget(
 *   id = "bef_flatpickr",
 *   label = @Translation("Date Picker with Flatpickr"),
 * )
 */
class FlatpickrDateBef extends FilterWidgetBase {

  use LoggerChannelTrait;
  use DateTimeFlatPickrWidgetTrait;

  /**
   * {@inheritdoc}
   */
  public static function isApplicable(mixed $handler = NULL, array $options = []): bool {
    /** @var \Drupal\views\Plugin\views\filter\FilterPluginBase $handler */
    $is_applicable = FALSE;

    if ((is_a($handler, 'Drupal\views\Plugin\views\filter\Date') || !empty($handler->date_handler)) && !$handler->isAGroup()) {
      $is_applicable = TRUE;
    }

    return $is_applicable;
  }

  /**
   * {@inheritdoc}
   */
  public function exposedFormAlter(array &$form, FormStateInterface $form_state): void {
    $field_id = $this->getExposedFilterFieldId();

    // Handle wrapper element added to exposed filters
    // in https://www.drupal.org/project/drupal/issues/2625136.
    $wrapper_id = $field_id . '_wrapper';
    if (!isset($form[$field_id]) && isset($form[$wrapper_id])) {
      $element = &$form[$wrapper_id][$field_id];
    }
    else {
      $element = &$form[$field_id];
    }

    parent::exposedFormAlter($form, $form_state);

    // Single Date API-based input element.
    $is_single_date = isset($element['value']['#type'])
      && 'date_text' == $element['value']['#type'];
    // Double Date-API-based input elements such as "in-between".
    $is_double_date = isset($element['min']) && isset($element['max'])
      && 'date_text' == $element['min']['#type']
      && 'date_text' == $element['max']['#type'];

    if ($is_single_date || $is_double_date) {
      if (isset($element['value'])) {
        $element['value']['#type'] = 'datetime_flatpickr';
        $element['value']['#dateFormat'] = 'Y-m-d';
      }
      else {
        // Both min and max share the same format.
        $element['min']['#type'] = 'datetime_flatpickr';
        $element['min']['#dateFormat'] = 'Y-m-d';
        $element['max']['#type'] = 'datetime_flatpickr';
        $element['max']['#dateFormat'] = 'Y-m-d';
      }

    }
    else {
      $fields = ['min', 'max', 'value'];
      if (count(array_intersect($fields, array_keys($element)))) {
        foreach ($fields as $field) {
          if (isset($element[$field])) {
            $element[$field]['#type'] = 'datetime_flatpickr';
            $element[$field]['#dateFormat'] = 'Y-m-d';
          }
        }
      }
      else {
        $element['#type'] = 'datetime_flatpickr';
        $element['#dateFormat'] = 'Y-m-d';
      }
    }
  }

}
